/*----------------------------------------------------------------------------
    ChucK Concurrent, On-the-fly Audio Programming Language
      Compiler and Virtual Machine

    Copyright (c) 2004 Ge Wang and Perry R. Cook.  All rights reserved.
      http://chuck.cs.princeton.edu/
      http://soundlab.cs.princeton.edu/

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    U.S.A.
-----------------------------------------------------------------------------*/

//-----------------------------------------------------------------------------
// file: chuck_bbq.cpp
// desc: implementation
//
// author: Ge Wang (gewang@cs.princeton.edu)
//         Perry R. Cook (prc@cs.princeton.edu)
//-----------------------------------------------------------------------------
#include "chuck_bbq.h"
#include <math.h>




//-----------------------------------------------------------------------------
// name: BBQ()
// desc: ...
//-----------------------------------------------------------------------------
BBQ::BBQ()
{
    m_digi_out = new DigitalOut;
    m_digi_in = new DigitalIn;
    m_midi_out = new MidiOut;
    m_midi_in = new MidiIn;
}




//-----------------------------------------------------------------------------
// name: ~BBQ()
// desc: ...
//-----------------------------------------------------------------------------
BBQ::~BBQ()
{
    SAFE_DELETE( m_digi_out );
    SAFE_DELETE( m_digi_in );
    SAFE_DELETE( m_midi_out );
    SAFE_DELETE( m_midi_in );
}




//-----------------------------------------------------------------------------
// name: initialize()
// desc: ...
//-----------------------------------------------------------------------------
BOOL__ BBQ::initialize( DWORD__ num_channels, DWORD__ sampling_rate,
                        DWORD__ bps, DWORD__ buffer_size, DWORD__ num_buffers )
{
    return Digitalio::initialize( 
        num_channels, sampling_rate, bps, buffer_size, num_buffers 
    );
}




//-----------------------------------------------------------------------------
// name: shutdown()
// desc: ...
//-----------------------------------------------------------------------------
void BBQ::shutdown()
{
    Digitalio::shutdown();
}




//-----------------------------------------------------------------------------
// name: digi_out()
// desc: digital out
//-----------------------------------------------------------------------------
DigitalOut & BBQ::digi_out()
{
    return *m_digi_out;
}




//-----------------------------------------------------------------------------
// name: digi_in()
// desc: digital in
//-----------------------------------------------------------------------------
DigitalIn & BBQ::digi_in()
{
    return *m_digi_in;
}




//-----------------------------------------------------------------------------
// name: midi_out()
// desc: midi out
//-----------------------------------------------------------------------------
MidiOut & BBQ::midi_out()
{
    return *m_midi_out;
}




//-----------------------------------------------------------------------------
// name: midi_in()
// desc: ...
//-----------------------------------------------------------------------------
MidiIn & BBQ::midi_in()
{
    return *m_midi_in;
}




//-----------------------------------------------------------------------------
// name: RangeMapper()
// desc: constructor
//-----------------------------------------------------------------------------
RangeMapper::RangeMapper( SAMPLE low_l, SAMPLE high_l,
                          SAMPLE low_r, SAMPLE high_r )
{
    SAMPLE dx = high_l - low_l;
    if( fabs( (double)dx ) < .000001 )
        m_slope = 0.0f;
    else
        m_slope = ( high_r - low_r ) / dx;

    m_inter = high_r - m_slope * high_l;
}




//-----------------------------------------------------------------------------
// name: map()
// desc: ...
//-----------------------------------------------------------------------------
SAMPLE RangeMapper::map( SAMPLE x )
{
    return m_slope * x + m_inter;
}
